import ast
import configparser
import os
from pathlib import Path

BEAUTILOG_INI_FILENAME = "beautilog.ini"


class Config:
    def __init__(self):
        self.config = configparser.ConfigParser()
        self.config.optionxform = str           # preserve case

        self.logger = {}
        self.file_logger = {}
        self.sections = {}
        self.load_defaults()

        ini_path = self.get_or_create_config_file()
        self._parse(ini_path)

    def load_defaults(self):
        default_config_path = Path(__file__).parent / BEAUTILOG_INI_FILENAME
        self._parse(default_config_path)

    def get_or_create_config_file(self) -> Path:
        cwd = Path.cwd()
        ini_path = cwd / BEAUTILOG_INI_FILENAME
        if not ini_path.exists():
            default_ini_path = Path(__file__).parent / BEAUTILOG_INI_FILENAME
            with open(default_ini_path, "r") as src, open(ini_path, "w") as dst:
                dst.write(src.read())

        return ini_path


    def _parse(self, ini_path: Path):
        self.config.read(ini_path)
        for section in self.config.sections():
            items = dict(self.config[section].items())
            # Try to parse values that look like lists or dicts
            for k, v in items.items():
                try:
                    items[k] = ast.literal_eval(v)
                except Exception:
                    items[k] = v

            if section == "logger":
                self.logger.update(items)
            elif section == "file_logger":
                self.file_logger.update(items)
            else:
                if section not in self.sections:
                    self.sections[section] = {}
                self.sections[section].update(items)

    def __getitem__(self, section):
        if section == "logger":
            return self.logger
        elif section == "file_logger":
            return self.file_logger
        return self.sections.get(section, {})

    def __repr__(self):
        return f"Config(logger={self.logger}, file_logger={self.file_logger}, sections={self.sections})"



from logging import Logger

# beautilog/__init__.py
from .beauti_logger import get_logger
from .constants import TERMINAL_COLORS
from .custom_handlers import ColoredConsoleHandler

# Initialize logger at import time
logger: Logger = get_logger()

# Optional: expose utility functions if needed later
__all__ = [
    "ColoredConsoleHandler",
    "TERMINAL_COLORS",
    "get_logger",
    "logger"
]



# fancy_log/logger_setup.py
import copy
import json
import logging
import os
import sys
from logging.handlers import RotatingFileHandler

from tqdm import tqdm

from .config import Config
from .constants import TERMINAL_COLORS
from .custom_handlers import ColoredConsoleHandler, ForceLevelFilter


def get_logger() -> logging.Logger:
    config = Config()
    logger = logging.getLogger()

    if config.logger.get("suppress_existing_loggers", False):
        logger.handlers.clear()

    for mute_loggger_name in config.logger.get("disabled_loggers", []):
        mute_logger = logging.getLogger(mute_loggger_name)
        mute_logger.setLevel(logging.CRITICAL)
        mute_logger.handlers.clear()

    formatter = logging.Formatter(
        '%(asctime)s : %(levelname)s : %(name)s : %(message)s',
        datefmt='%m/%d/%y %I:%M:%S %p'
    )

    if config.logger.get("save_to_file", True):
        file_handler = RotatingFileHandler(
            config.file_logger.get("log_file_path", "fancy-run.log"),
            mode='a',
            maxBytes=config.file_logger.get("max_bytes", 10485760),  # Default 100 MB
            backupCount=config.file_logger.get("backup_count", 5),
            encoding='utf-8'
        )
        file_handler.setFormatter(formatter)
        file_handler.setLevel(getattr(logging, config.file_logger.get("log_level", "DEBUG").upper(), logging.DEBUG))
        logger.addHandler(file_handler)

    def write(self, x):
        if len(x.rstrip()) > 0:
            tqdm.write(x, file=self.file, end='')
    out_stream = type("TqdmStream", (), {'file': sys.stdout, 'write': write})()

    console_handler = ColoredConsoleHandler(out_stream, config.sections.get("level_colors", {}))
    console_handler.setFormatter(formatter)
    console_handler.setLevel(getattr(logging, config.sections.get("log_level", "INFO").upper(), logging.INFO))

    logger.addHandler(console_handler)
    logger.setLevel(getattr(logging, config.logger.get("log_level", "INFO").upper(), logging.INFO))

    for level_name, level_value in config.sections.get("custom_levels", {}).items():
        logging.addLevelName(level_value, level_name.upper())
        setattr(logging, level_name.upper(), level_value)
        setattr(logger, level_name.upper(), level_value)
        setattr(logger, level_name.lower(), lambda msg, level=level_value: logger.log(level, msg))


    for logger_name, level in config.sections.get("redirected_loggers", {}).items():
        redirected_logger = logging.getLogger(logger_name)
        stream_handler = logging.StreamHandler(out_stream)
        stream_handler.addFilter(ForceLevelFilter(getattr(logging, level.upper(), logging.INFO)))
        stream_handler.setFormatter(formatter)
        redirected_logger.addHandler(stream_handler)

    return logger
